/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Intbox;
import org.zkoss.zul.Listheader;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Row;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.Ciudad;
import com.rjconsultores.ventaboletos.entidad.Estado;
import com.rjconsultores.ventaboletos.service.EstadoService;
import com.rjconsultores.ventaboletos.utilerias.ApplicationProperties;
import com.rjconsultores.ventaboletos.utilerias.CustomEnum;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderCiudad;

/**
 * 
 * @author MCosso
 */
@Controller("busquedaCiudadController")
@Scope("prototype")
public class BusquedaCiudadController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	@Autowired
	private transient PagedListWrapper<Ciudad> plwCiudad;
	@Autowired
	private EstadoService estadoService;
	private MyListbox ciudadList;
	private Paging pagingCiudad;
	private List<Estado> lsEstado;
	private Textbox txtNombre;
	private Textbox txtEquivalencia;
	private Intbox txtCodMunicipio;
	private Intbox txtCodIBGE;
	private Combobox cmbEstado;
	private Row rowEquivalencia;
	private Listheader headerEquivalencia;
	private Intbox txtZonaHoraria;

	public List<Estado> getLsEstado() {
		return lsEstado;
	}

	public void setLsEstado(List<Estado> lsEstado) {
		this.lsEstado = lsEstado;
	}

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEstado = estadoService.obtenerTodos();

		super.doAfterCompose(comp);

		ciudadList.setItemRenderer(new RenderCiudad());
		ciudadList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				Ciudad ciu = (Ciudad) ciudadList.getSelected();
				verCiudad(ciu);
			}
		});

		refreshLista();

		if (!ApplicationProperties.getInstance().isCustomHabilitado(CustomEnum.MOSTRAR_EQUIVALENCIA.getDescricao())) {
			rowEquivalencia.setVisible(Boolean.FALSE);
			headerEquivalencia.setVisible(Boolean.FALSE);
		}

		txtNombre.focus();
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void verCiudad(Ciudad ci) {
		if (ci == null) {
			return;
		}

		Map args = new HashMap();
		args.put("ciudad", ci);
		args.put("ciudadList", ciudadList);

		openWindow("/gui/catalogos/editarCiudad.zul",
				Labels.getLabel("editarCiudadController.window.title"), args, MODAL);
	}

	private void refreshLista() {
		HibernateSearchObject<Ciudad> ciudadBusqueda =
				new HibernateSearchObject<Ciudad>(Ciudad.class, pagingCiudad.getPageSize());

		ciudadBusqueda.addFilterEqual(ACTIVO, true);
		ciudadBusqueda.addFilterLike("nombciudad", txtNombre.getText().trim().concat("%"));
		ciudadBusqueda.addFilterNotEqual("ciudadId", -1);

		if (!txtEquivalencia.getText().isEmpty()) {
			ciudadBusqueda.addFilterLike("equivalenciaId", "%" + txtEquivalencia.getText().trim().concat("%"));
		}
		
		if (!txtCodMunicipio.getText().isEmpty()) {
			ciudadBusqueda.addFilterEqual("codmunicipio",txtCodMunicipio.getValue());
		}
		
		if (!txtCodIBGE.getText().isEmpty()) {
			ciudadBusqueda.addFilterEqual("codibge", txtCodIBGE.getValue());
		}
		Comboitem cbiEstado = cmbEstado.getSelectedItem();

		if (cbiEstado != null) {
			Estado estado = (Estado) cbiEstado.getValue();
			ciudadBusqueda.addFilterEqual("estado", estado);
		}

		if (txtZonaHoraria.getValue() != null) {
			ciudadBusqueda.addFilterEqual("zonaHoraria", txtZonaHoraria.getValue());
		}

		ciudadBusqueda.addSortAsc("nombciudad");

		plwCiudad.init(ciudadBusqueda, ciudadList, pagingCiudad);

		if (ciudadList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaCiudadController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) {
		verCiudad(new Ciudad());
	}
}
