/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import com.rjconsultores.ventaboletos.entidad.Estado;
import com.rjconsultores.ventaboletos.entidad.Pais;
import com.rjconsultores.ventaboletos.service.PaisService;
import com.rjconsultores.ventaboletos.utilerias.ApplicationProperties;
import com.rjconsultores.ventaboletos.utilerias.CustomEnum;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderEstado;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Listheader;
import org.zkoss.zul.Messagebox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Row;
import org.zkoss.zul.Textbox;

/**
 * 
 * @author Rafius
 */
@Controller("busquedaEstadoController")
@Scope("prototype")
public class BusquedaEstadoController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	@Autowired
	private PaisService paisService;
	@Autowired
	private transient PagedListWrapper<Estado> plwEstado;
	private MyListbox estadoList;
	private Paging pagingEstado;
	private Textbox txtNombre;
	private Textbox txtEquivalencia;
	private Combobox cmbPais;
	private List<Pais> lsPais;
	private Row rowEquivalencia;
	private Listheader headerEquivalencia;

	public List<Pais> getLsPais() {
		return lsPais;
	}

	public void setLsPais(List<Pais> lsPais) {
		this.lsPais = lsPais;
	}

	@Override
	public void doAfterCompose(Component comp) throws Exception {

		lsPais = paisService.obtenerTodos();
		super.doAfterCompose(comp);

		estadoList.setItemRenderer(new RenderEstado());
		estadoList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				Estado est = (Estado) estadoList.getSelected();
				verPeriodo(est);
			}
		});

		refreshLista();

		if (!ApplicationProperties.getInstance().isCustomHabilitado(CustomEnum.MOSTRAR_EQUIVALENCIA.getDescricao())) {
			rowEquivalencia.setVisible(Boolean.FALSE);
			headerEquivalencia.setVisible(Boolean.FALSE);
		}

		txtNombre.focus();
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void verPeriodo(Estado e) {
		if (e == null) {
			return;
		}

		Map args = new HashMap();
		args.put("estado", e);
		args.put("estadoList", estadoList);

		openWindow("/gui/catalogos/editarEstado.zul",
				Labels.getLabel("editarEstadoController.window.title"), args, MODAL);
	}

	private void refreshLista() throws InterruptedException {
		HibernateSearchObject<Estado> estadoBusqueda =
				new HibernateSearchObject<Estado>(Estado.class, pagingEstado.getPageSize());

		estadoBusqueda.addFilterLike("nombestado", "%" + txtNombre.getText().trim().concat("%"));

		Comboitem itemPais = cmbPais.getSelectedItem();
		if (itemPais != null) {
			Pais pais = (Pais) itemPais.getValue();
			estadoBusqueda.addFilterEqual("pais", pais);
		}

		if (!txtEquivalencia.getText().isEmpty()) {
			estadoBusqueda.addFilterLike("equivalenciaId", txtEquivalencia.getText().trim().concat("%"));
		}

		estadoBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);
		estadoBusqueda.addSortAsc("nombestado");
		estadoBusqueda.addFilterNotEqual("estadoId", -1);

		plwEstado.init(estadoBusqueda, estadoList, pagingEstado);

		if (estadoList.getData().length == 0) {
			Messagebox.show(
					Labels.getLabel("MSG.ningunRegistro"),
					Labels.getLabel("busquedaEstadoController.window.title"),
					Messagebox.OK, Messagebox.INFORMATION);
		}
	}

	public void onClick$btnRefresh(Event ev) throws InterruptedException {
		refreshLista();
	}

	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) throws InterruptedException {
		verPeriodo(new Estado());
	}
}
