package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Bandbox;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;
import org.zkoss.zul.Radio;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioRemarcacaoPassagensTransferencia;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEstandar;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderPuntoVentaSimple;
import com.trg.search.Filter;

@Controller("relatorioRemarcacaoPassagensTransferenciaController")
@Scope("prototype")
public class RelatorioRemarcacaoPassagensTransferenciaController extends MyGenericForwardComposer {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static final String TODOS_VALUE = new Integer(-1).toString();
	private static final String TODOS = "TODAS";
	@Autowired
	private EmpresaService empresaService;
	private List<Empresa> lsEmpresa;
	private MyComboboxEstandar cmbEmpresa;
	private Datebox datInicial;
	private Datebox datFinal;
	@Autowired
	private DataSource dataSourceRead;
	private MyListbox puntoVentaList;;
	private MyListbox puntoVentaSelList;
	private Paging pagingPuntoVenta;
	private Bandbox bbPesquisaPuntoVenta;
	@Autowired
	private transient PagedListWrapper<PuntoVenta> plwPuntoVenta;
	private ArrayList<PuntoVenta> lsNumPuntoVenta = new ArrayList<PuntoVenta>();

	private Textbox txtPalavraPesquisa;
	private Radio rdTipoTransferencia;
	private Radio rdTipoCheckinAmbos;
	private Radio rdTipoCheckinReativado;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);
		lsEmpresa = empresaService.obtenerTodos();
		puntoVentaList.setItemRenderer(new RenderPuntoVentaSimple());
		puntoVentaSelList.setItemRenderer(new RenderPuntoVentaSimple());
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void executarRelatorio() throws Exception {
		Relatorio relatorio;

		Map<String, Object> parametros = new HashMap<String, Object>();

		Timestamp dataInicio = new Timestamp((DateUtil.inicioFecha((java.util.Date) this.datInicial.getValue()).getTime()));
		Timestamp dataFinal = new Timestamp((DateUtil.fimFecha((java.util.Date) this.datFinal.getValue()).getTime()));

		parametros.put("DATA_INICIAL", dataInicio);
		parametros.put("DATA_FINAL", dataFinal);
		parametros.put("CONSIDERAR_APENAS_TRANSFERENCIA", rdTipoTransferencia.isChecked());
		parametros.put("CONSIDERAR_APENAS_REATIVADO", rdTipoCheckinReativado.isChecked());
		parametros.put("CONSIDERAR_AMBOS", rdTipoCheckinAmbos.isChecked());
		Empresa empresa =null;
		if(cmbEmpresa.getSelectedItem() != null ){
			empresa = (Empresa) cmbEmpresa.getSelectedItem().getValue();
		}
		 
		lsNumPuntoVenta = new ArrayList(Arrays.asList(puntoVentaSelList.getData()));

		if (lsNumPuntoVenta.size() > 0) {
			String puntoVentaExpression = null;
			for (PuntoVenta p : lsNumPuntoVenta) {
				if (lsNumPuntoVenta.indexOf(p) == 0) {
					puntoVentaExpression = p.getPuntoventaId().toString();
					parametros.put("AGENCIA", p.getNombpuntoventa());
				} else {
					puntoVentaExpression += ", " + p.getPuntoventaId().toString();
				}
			}
			parametros.put("PUNTOVENTA_ID", puntoVentaExpression);
			parametros.put("PUNTOVENTA", puntoVentaExpression);
			
		} else {
			parametros.put("PUNTOVENTA_ID", TODOS_VALUE);
			parametros.put("PUNTOVENTA", TODOS);
			parametros.put("AGENCIA", TODOS);
		}

		
		if (empresa != null) {
			parametros.put("EMPRESA", empresa.getNombempresa());
			parametros.put("EMPRESA_ID", empresa.getEmpresaId());
		} else {
			parametros.put("EMPRESA", "TODOS");
		}
		parametros.put("TITULO", Labels.getLabel("relatorioRemarcacaoPassagensTransferenciaController.window.title"));

		relatorio = new RelatorioRemarcacaoPassagensTransferencia(parametros, dataSourceRead.getConnection());

		Map args = new HashMap();
		args.put("relatorio", relatorio);

		openWindow("/component/reportView.zul",
				Labels.getLabel("relatorioRemarcacaoPassagensTransferenciaController.window.title"), args, MODAL);
	}

	public void onSelect$puntoVentaList(Event ev) {

	}

	public void onClick$btnLimpar(Event ev) {
		limparPesquisaAgencia();
	}

	public void onClick$btnPesquisa(Event ev) {
		executarPesquisa();
	}

	private void executarPesquisa() {
		HibernateSearchObject<PuntoVenta> puntoVentaBusqueda = new HibernateSearchObject<PuntoVenta>(PuntoVenta.class,
				pagingPuntoVenta.getPageSize());

		puntoVentaBusqueda.addFilterOr(Filter.like("nombpuntoventa", "%" + txtPalavraPesquisa.getText().trim().toUpperCase().concat("%")), Filter.like("numPuntoVenta", "%" + txtPalavraPesquisa.getText().trim().toUpperCase().concat("%")));

		puntoVentaBusqueda.addSortAsc("nombpuntoventa");

		puntoVentaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);

		plwPuntoVenta.init(puntoVentaBusqueda, puntoVentaList, pagingPuntoVenta);

		if (puntoVentaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("relatorioRemarcacaoPassagensTransferenciaController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	private void limparPesquisaAgencia() {
		puntoVentaList.clearSelection();

		lsNumPuntoVenta.clear();
		this.bbPesquisaPuntoVenta.setValue("");
	}

	public void onDoubleClick$puntoVentaList(Event ev) {

		PuntoVenta puntoVentaSel = (PuntoVenta) puntoVentaList.getSelected();
		puntoVentaSelList.addItemNovo(puntoVentaSel);

	}

	public void onDoubleClick$puntoVentaSelList(Event ev) {

		PuntoVenta puntoVentaSel = (PuntoVenta) puntoVentaSelList.getSelected();
		puntoVentaSelList.removeItem(puntoVentaSel);

	}

	public void onClick$btnExecutarRelatorio(Event ev) throws Exception {
		executarRelatorio();
	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	public MyListbox getPuntoVentaList() {
		return puntoVentaList;
	}

	public void setPuntoVentaList(MyListbox puntoVentaList) {
		this.puntoVentaList = puntoVentaList;
	}

	public Paging getPagingPuntoVenta() {
		return pagingPuntoVenta;
	}

	public void setPagingPuntoVenta(Paging pagingPuntoVenta) {
		this.pagingPuntoVenta = pagingPuntoVenta;
	}

	public Textbox getTxtPalavraPesquisa() {
		return txtPalavraPesquisa;
	}

	public void setTxtPalavraPesquisa(Textbox txtPalavraPesquisa) {
		this.txtPalavraPesquisa = txtPalavraPesquisa;
	}

	public Bandbox getBbPesquisaPuntoVenta() {
		return bbPesquisaPuntoVenta;
	}

	public void setBbPesquisaPuntoVenta(Bandbox bbPesquisaPuntoVenta) {
		this.bbPesquisaPuntoVenta = bbPesquisaPuntoVenta;
	}

	public MyListbox getPuntoVentaSelList() {
		return puntoVentaSelList;
	}

	public void setPuntoVentaSelList(MyListbox puntoVentaSelList) {
		this.puntoVentaSelList = puntoVentaSelList;
	}
}
