package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Bandbox;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioTrechoVendido;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderPuntoVentaSimple;
import com.trg.search.Filter;

@Controller("relatorioTrechoVendidoController")
@Scope("prototype")
public class RelatorioTrechoVendidoController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	private static final String TODOS = "TODAS";
	private static final String TODOS_VALUE = new Integer(-1).toString();

	private static Logger log = LogManager.getLogger(RelatorioTrechoVendidoController.class);

	@Autowired
	private DataSource dataSourceRead;

	@Autowired
	private EmpresaService empresaService;

	private ArrayList<PuntoVenta> lsNumPuntoVenta = new ArrayList<PuntoVenta>();

	@Autowired
	private transient PagedListWrapper<PuntoVenta> plwPuntoVenta;
	private MyListbox puntoVentaList;
	private Paging pagingPuntoVenta;
	private Bandbox bbPesquisaPuntoVenta;
	private Textbox txtPalavraPesquisa;
	private Datebox datInicial;
	private Datebox datFinal;

	private MyListbox puntoVentaSelList;

	private Combobox cmbEmpresa;
	private List<Empresa> lsEmpresa;

	private void executarRelatorio() throws Exception {

		Map<String, Object> parametros = new HashMap<String, Object>();

		Timestamp dataInicio = new Timestamp(((java.util.Date) this.datInicial.getValue()).getTime());
		Timestamp dataFinal = new Timestamp(((java.util.Date) this.datFinal.getValue()).getTime());

		GregorianCalendar auxDataInicio = (GregorianCalendar) GregorianCalendar.getInstance();
		auxDataInicio.setTimeInMillis(dataInicio.getTime());

		int year = auxDataInicio.get(Calendar.YEAR);
		int month = auxDataInicio.get(Calendar.MONTH);
		int date = auxDataInicio.get(Calendar.DATE);

		int hourOfDay = 00;
		int minute = 00;
		int second = 00;

		auxDataInicio.set(year, month, date, hourOfDay, minute, second);
		dataInicio = new Timestamp(auxDataInicio.getTimeInMillis());

		GregorianCalendar auxDataFinal = (GregorianCalendar) GregorianCalendar.getInstance();
		auxDataFinal.setTimeInMillis(dataFinal.getTime());

		year = auxDataFinal.get(Calendar.YEAR);
		month = auxDataFinal.get(Calendar.MONTH);
		date = auxDataFinal.get(Calendar.DATE);

		hourOfDay = 23;
		minute = 59;
		second = 59;

		auxDataFinal.set(year, month, date, hourOfDay, minute, second);
		dataFinal = new Timestamp(auxDataFinal.getTimeInMillis());

		parametros.put("DATA_INICIO", dataInicio);
		parametros.put("DATA_FINAL", dataFinal);

		lsNumPuntoVenta = new ArrayList(Arrays.asList(puntoVentaSelList.getData()));

		if (lsNumPuntoVenta.size() > 0) {
			String puntoVentaExpression = null;
			for (PuntoVenta p : lsNumPuntoVenta) {
				if (lsNumPuntoVenta.indexOf(p) == 0) {
					puntoVentaExpression = p.getPuntoventaId().toString();
				} else {
					puntoVentaExpression += ", " + p.getPuntoventaId().toString();
				}
			}
			parametros.put("PUNTOVENTA_ID", puntoVentaExpression);
			parametros.put("PUNTOVENTA", puntoVentaExpression);
		} else {
			parametros.put("PUNTOVENTA_ID", TODOS_VALUE);
			parametros.put("PUNTOVENTA", TODOS);
		}

		String empresa = "";
		try {
			Empresa e = (Empresa) cmbEmpresa.getSelectedItem().getValue();
			if (!e.equals(null)) {
				empresa = e.getNombempresa();
			}
		} catch (NullPointerException e) {
			empresa = "";
		} catch (Exception e) {
			log.error(e.getMessage());
		}
		parametros.put("NOMBEMPRESA", empresa);

		String empresaSearch = "%" + empresa + "%";
		parametros.put("NOMBEMPRESA_SEARCH", empresaSearch);

		Date now = new Date();
		parametros.put("NOW", now);

		Relatorio relatorio = new RelatorioTrechoVendido(parametros, dataSourceRead.getConnection());

		Map args = new HashMap();
		args.put("relatorio", relatorio);

		openWindow("/component/reportView.zul",
				Labels.getLabel("relatorioTrechoVendidoController.window.title"), args, MODAL);
	}

	public void onDoubleClick$puntoVentaList(Event ev) {

		PuntoVenta puntoVentaSel = (PuntoVenta) puntoVentaList.getSelected();
		puntoVentaSelList.addItemNovo(puntoVentaSel);

	}

	public void onDoubleClick$puntoVentaSelList(Event ev) {

		PuntoVenta puntoVentaSel = (PuntoVenta) puntoVentaSelList.getSelected();
		puntoVentaSelList.removeItem(puntoVentaSel);

	}

	public void onSelect$puntoVentaList(Event ev) {

	}

	public void onClick$btnLimpar(Event ev) {
		limparPesquisaAgencia();
	}

	public void onClick$btnPesquisa(Event ev) {
		executarPesquisa();
	}

	public void onClick$btnExecutarRelatorio(Event ev) throws Exception {
		executarRelatorio();
	}

	private void limparPesquisaAgencia() {
		puntoVentaList.clearSelection();

		lsNumPuntoVenta.clear();
		this.bbPesquisaPuntoVenta.setValue("");
	}

	private void executarPesquisa() {
		HibernateSearchObject<PuntoVenta> puntoVentaBusqueda =
				new HibernateSearchObject<PuntoVenta>(PuntoVenta.class,
						pagingPuntoVenta.getPageSize());

		puntoVentaBusqueda.addFilterOr(Filter.like("nombpuntoventa", "%" + txtPalavraPesquisa.getText().trim().toUpperCase().concat("%")), Filter.like("numPuntoVenta", "%" + txtPalavraPesquisa.getText().trim().toUpperCase().concat("%")));

		puntoVentaBusqueda.addSortAsc("nombpuntoventa");

		puntoVentaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);

		plwPuntoVenta.init(puntoVentaBusqueda, puntoVentaList, pagingPuntoVenta);

		if (puntoVentaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("relatorioReceitaDiariaAgenciaController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);
		lsEmpresa = empresaService.obtenerTodos();

		puntoVentaList.setItemRenderer(new RenderPuntoVentaSimple());
		puntoVentaSelList.setItemRenderer(new RenderPuntoVentaSimple());
	}

	public DataSource getDataSource() {
		return dataSourceRead;
	}

	public void setDataSource(DataSource dataSource) {
		this.dataSourceRead = dataSource;
	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

}
